#!/usr/bin/wish

proc Scrolled_Canvas { life args } {
    toplevel $life

    wm title $life "TkLife Canvas"
    wm protocol $life WM_DELETE_WINDOW "destroy $life"

    eval {canvas $life.canvas -xscrollcommand [list $life.xscroll set] \
	    -yscrollcommand [list $life.yscroll set] \
	    -highlightthickness 0 -borderwidth 0} $args
    
    scrollbar $life.xscroll -orient horizontal \
	    -command [list $life.canvas xview]
    scrollbar $life.yscroll -orient vertical \
	    -command [list $life.canvas yview]

    grid $life.canvas $life.yscroll -sticky news
    grid $life.xscroll -sticky ew
    grid rowconfigure $life 0 -weight 1
    grid columnconfigure $life 0 -weight 1
    return $life.canvas
}

proc Life_Begin { x_n y_n } {

    global mod_value
    global current_iteration
    global number_of_iterations
    global run_status

    set number_of_iterations [ expr $x_n*$y_n*1000 ]

    while { $current_iteration <= $number_of_iterations } {

	set x [expr rand()]
	set y [expr rand()]

	set x [expr $x*$x_n]
	set y [expr $y*$y_n]

	set xi [expr round($x)]
	set yi [expr round($y)]

	set r [expr rand()*15]
	set g [expr rand()*15]
	set b [expr rand()*15]

	set ri [expr round($r)]
	set gi [expr round($g)]
	set bi [expr round($b)]

	set rgb_R [format "#%01x" $ri]
	set rgb_G [format "#%01x" $gi]
	set rgb_B [format "#%01x" $bi]

	regsub -all "\#" $rgb_R "" rgb_R
	regsub -all "\#" $rgb_G "" rgb_G
	regsub -all "\#" $rgb_B "" rgb_B

	if { $rgb_R == "0" || $rgb_R == "1" || $rgb_R == "2" || $rgb_R == "3" } {
		set rgb_R "0"
	}
	if { $rgb_R == "4" || $rgb_R == "5" || $rgb_R == "6" || $rgb_R == "7" } {
                set rgb_R "5"
        }
	if { $rgb_R == "8" || $rgb_R == "9" || $rgb_R == "a" || $rgb_R == "b" } {
                set rgb_R "a"
        }
	if { $rgb_R == "c" || $rgb_R == "d" || $rgb_R == "e" || $rgb_R == "f" } {
                set rgb_R "f"
        }

	if { $rgb_G == "0" || $rgb_G == "1" || $rgb_G == "2" || $rgb_G == "3" } {
                set rgb_G "0"
        }
        if { $rgb_G == "4" || $rgb_G == "5" || $rgb_G == "6" || $rgb_G == "7" } {
                set rgb_G "5"
        }
        if { $rgb_G == "8" || $rgb_G == "9" || $rgb_G == "a" || $rgb_G == "b" } {
                set rgb_G "a"
        }
        if { $rgb_G == "c" || $rgb_G == "d" || $rgb_G == "e" || $rgb_G == "f" } {
                set rgb_G "f"
        }

	if { $rgb_B == "0" || $rgb_B == "1" || $rgb_B == "2" || $rgb_B == "3" } {
                set rgb_B "0"
        }
        if { $rgb_B == "4" || $rgb_B == "5" || $rgb_B == "6" || $rgb_B == "7" } {
                set rgb_B "5"
        }
        if { $rgb_B == "8" || $rgb_B == "9" || $rgb_B == "a" || $rgb_B == "b" } {
                set rgb_B "a"
        }
        if { $rgb_B == "c" || $rgb_B == "d" || $rgb_B == "e" || $rgb_B == "f" } {
                set rgb_B "f"
        }

	set rgb "#$rgb_R$rgb_G$rgb_B"

	update

	set s [expr fmod($current_iteration,$mod_value)]

	if { $xi != 0 && $yi != 0 && $s == 0 && $run_status == "RUN" } {
	    Metamorphic_Node1 $xi $yi $rgb
	}

	if { $xi > 1 && $yi > 1 && $s != 0 && $run_status == "RUN" } {
	    if { $xi < $x_n && $yi < $y_n } {
		Metamorphic_Node2 $xi $yi
	    }
	}

	if { $run_status != "RUN" } {
	    after 500
	    puts " PAUSE... type \"RUN\" to continue ... "
	    update
	}

	incr current_iteration
    }
    bell
}

proc Metamorphic_Node1 { xi yi rgb } {

    global node_array
    .life.canvas itemconfigure $node_array($xi,$yi) -fill $rgb -outline $rgb
    set tag_list [.life.canvas gettags $node_array($xi,$yi)]
    .life.canvas addtag "painted" withtag $tag_list

}

proc Metamorphic_Node2 { xi yi } {

    global node_array

    set xl [expr $xi - 1]
    set xr [expr $xi + 1]
    set yu [expr $yi - 1]
    set yd [expr $yi + 1]

    set inner_node $node_array($xi,$yi)
    set upper_node $node_array($xi,$yu)
    set lower_node $node_array($xi,$yd)
    set left_node  $node_array($xl,$yi)
    set right_node $node_array($xr,$yi)

    set left_up    $node_array($xl,$yu)
    set left_down  $node_array($xl,$yd)
    set right_up   $node_array($xr,$yu)
    set right_down $node_array($xr,$yd)

    ### GET COLOR INFO FOR NODES ###

    set tag_list_in [.life.canvas gettags $inner_node]
    set tag_list_up [.life.canvas gettags $upper_node]
    set tag_list_lw [.life.canvas gettags $lower_node]
    set tag_list_lf [.life.canvas gettags  $left_node]
    set tag_list_rt [.life.canvas gettags $right_node]

    set color_in [lindex [.life.canvas itemconfig $inner_node -fill] 4]
    set color_up [lindex [.life.canvas itemconfig $upper_node -fill] 4]
    set color_lw [lindex [.life.canvas itemconfig $lower_node -fill] 4]
    set color_lf [lindex [.life.canvas itemconfig  $left_node -fill] 4]
    set color_rt [lindex [.life.canvas itemconfig $right_node -fill] 4]

    ### END OF COLOR INFO ###

    ### CONDITION A ###
    if { [lindex $tag_list_in 1] == "painted" && [lindex $tag_list_up 1] == "painted" &&  [lindex $tag_list_lw 1] == "painted" && [lindex $tag_list_lf 1] == "painted" && [lindex $tag_list_rt 1] == "painted"} {
	### CONDITION B ###
	if { $color_in == $color_up && $color_in == $color_lw && $color_in ==  $color_lf && $color_in ==  $color_rt } {
	    .life.canvas itemconfigure $left_up    -fill $color_in -outline $color_in
	    .life.canvas itemconfigure $left_down  -fill $color_in -outline $color_in
	    .life.canvas itemconfigure $right_up   -fill $color_in -outline $color_in
	    .life.canvas itemconfigure $right_down -fill $color_in -outline $color_in
	    update
	} else {
	.life.canvas itemconfigure $inner_node -fill $color_in -outline $color_in
	.life.canvas itemconfigure $upper_node -fill $color_in -outline $color_in
	.life.canvas itemconfigure $lower_node -fill $color_in -outline $color_in
	.life.canvas itemconfigure  $left_node -fill $color_in -outline $color_in
	.life.canvas itemconfigure $right_node -fill $color_in -outline $color_in
	update
	}
    }
}

proc Life_Plot { x_n y_n z_n } {

    set  l 1
    set y1 5

    while { $l <= $y_n } {
	Plot_X $l $y1 $x_n $z_n
	incr l
	set y1 [expr $y1 + $z_n + 2]
    }
    Life_Begin $x_n $y_n
}

proc Plot_X { l y1 x_n z_n } {

    global node_array

    set  k 1
    set x1 5
    while { $k <= $x_n } {
	set x2 [expr $x1 + $z_n]
	set y2 [expr $y1 + $z_n]
	set node_tag "node_$l\_$k"
	set node_array($k,$l) [.life.canvas create rect $x1 $y1 $x2 $y2 \
		-fill "#000" -outline "#fff" -tag "$node_tag"]
	update
	incr k
	set x1 [expr $x1 + $z_n + 2]
    }
}

proc Get_Parameters { argv } {

    puts "$argv PARAMETERS"

    if { [lindex $argv 0] != "" } {
	set x_n [lindex $argv 0]
    } else {
	set x_n 100
    }
    if { [lindex $argv 1] != "" } {
	set y_n [lindex $argv 1]
    } else {
	set y_n 75
    }
    if { [lindex $argv 2] != "" } {
	set z_n [lindex $argv 2]
    } else {
	set z_n 7
    }

    puts "X: $x_n  Y: $y_n  Z: $z_n"

    set scrollmax_x [expr ($x_n*$z_n + 20)]
    set scrollmax_y [expr ($y_n*$z_n + 20)]
    Scrolled_Canvas .life -width 700 -height 550 -bg black \
	    -scrollregion "0 0 $scrollmax_x $scrollmax_y"

    Life_Plot $x_n $y_n $z_n
}

wm title . "TkLife"
wm protocol . WM_DELETE_WINDOW "destroy ."
. config -bg black

global mod_value
global current_iteration
global number_of_iterations
global run_status

set mod_value 15
set current_iteration 0
set run_status "RUN"

label .curr_iteration -text "Current Iteration: " -bg "#003" -fg "#fff"
entry .curr_iteration_entry -relief sunken -textvariable current_iteration \
	-bg "#003" -fg "#fff" -width 24 -insertbackground lightyellow
grid .curr_iteration .curr_iteration_entry

label .numb_iteration -text "Number of Iterations: " -bg "#003" -fg "#fff"
entry .numb_iteration_entry -relief sunken -textvariable number_of_iterations \
	-bg "#003" -fg "#fff" -width 24 -insertbackground lightyellow
grid .numb_iteration .numb_iteration_entry

label .mut_value -text "Mutation K Value: " -bg "#003" -fg "#fff"
entry .mut_value_entry -relief sunken -textvariable mod_value \
	-bg "#003" -fg "#fff" -width 24 -insertbackground lightyellow
grid .mut_value .mut_value_entry

label .run_value -text "Run Status (\"RUN\" to run): " -bg "#003" -fg "#fff"
entry .run_value_entry -relief sunken -textvariable run_status \
	-bg "#003" -fg "#fff" -width 24 -insertbackground lightyellow
grid .run_value .run_value_entry

Get_Parameters $argv
